@@ -1,10 +1,10 @@
 """SatNOGS DB django base Forms class"""
 from bootstrap_modal_forms.forms import BSModalModelForm
 from django.core.exceptions import ValidationError
-from django.forms import NumberInput, TextInput
-from django.utils.translation import ugettext_lazy as _
+from django.forms import TextInput
+from django.utils.translation import gettext_lazy as _
 
-from db.base.models import Satellite, Transmitter, TransmitterEntry
+from db.base.models import SatelliteEntry, Transmitter, TransmitterEntry
 
 
 def existing_uuid(value):
@@ -19,7 +19,7 @@ def existing_uuid(value):
         ) from error
 
 
-class TransmitterModelForm(BSModalModelForm):  # pylint: disable=too-many-ancestors
+class TransmitterCreateForm(BSModalModelForm):  # pylint: disable=too-many-ancestors
     """Model Form class for TransmitterEntry objects"""
     class Meta:
         model = TransmitterEntry
@@ -57,16 +57,18 @@ class TransmitterUpdateForm(BSModalModelForm):  # pylint: disable=too-many-ances
         }
 
 
-class SatelliteModelForm(BSModalModelForm):
+class SatelliteCreateForm(BSModalModelForm):
     """Form that uses django-bootstrap-modal-forms for satellite editing"""
     class Meta:
-        model = Satellite
+        model = SatelliteEntry
         fields = [
-            'norad_cat_id', 'name', 'names', 'operator', 'status', 'description', 'countries',
-            'website', 'dashboard_url', 'launched', 'deployed', 'decayed', 'image'
+            'norad_cat_id', 'norad_follow_id', 'name', 'names', 'description', 'operator',
+            'status', 'countries', 'website', 'dashboard_url', 'launched', 'deployed', 'decayed',
+            'image', 'citation'
         ]
         labels = {
             'norad_cat_id': _('Norad ID'),
+            'norad_follow_id': _('Followed Norad ID'),
             'names': _('Other names'),
             'countries': _('Countries of Origin'),
             'launched': _('Launch Date'),
@@ -76,4 +78,28 @@ class SatelliteModelForm(BSModalModelForm):
             'dashboard_url': _('Dashboard URL'),
             'operator': _('Owner/Operator'),
         }
-        widgets = {'norad_cat_id': NumberInput(attrs={'readonly': True}), 'names': TextInput()}
+        widgets = {'names': TextInput()}
+
+
+class SatelliteUpdateForm(BSModalModelForm):
+    """Form that uses django-bootstrap-modal-forms for satellite editing"""
+    class Meta:
+        model = SatelliteEntry
+        fields = [
+            'norad_cat_id', 'norad_follow_id', 'name', 'names', 'description', 'operator',
+            'status', 'countries', 'website', 'dashboard_url', 'launched', 'deployed', 'decayed',
+            'image', 'citation'
+        ]
+        labels = {
+            'norad_cat_id': _('Norad ID'),
+            'norad_follow_id': _('Followed Norad ID'),
+            'names': _('Other names'),
+            'countries': _('Countries of Origin'),
+            'launched': _('Launch Date'),
+            'deployed': _('Deploy Date'),
+            'decayed': _('Re-entry Date'),
+            'description': _('Description'),
+            'dashboard_url': _('Dashboard URL'),
+            'operator': _('Owner/Operator'),
+        }
+        widgets = {'names': TextInput()}
