@@ -1,5 +1,6 @@
 """Django database model for SatNOGS DB"""
 import logging
+import re
 from os import path
 from uuid import uuid4
 
@@ -11,10 +12,11 @@ from django.core.exceptions import ValidationError
 from django.core.validators import MaxLengthValidator, MaxValueValidator, MinLengthValidator, \
     MinValueValidator, URLValidator
 from django.db import models
-from django.db.models import OuterRef, Subquery
+from django.db.models import OuterRef, Q, Subquery
 from django.utils.timezone import now
 from django_countries.fields import CountryField
 from markdown import markdown
+from nanoid import generate
 from shortuuidfield import ShortUUIDField
 
 LOGGER = logging.getLogger('db')
@@ -79,9 +81,47 @@ class Operator(models.Model):
         return self.name
 
 
-class Satellite(models.Model):
-    """Model for all the satellites."""
-    norad_cat_id = models.PositiveIntegerField()
+def validate_sat_id(value):
+    """Validate a Satellite Identifier"""
+    if not re.compile(r'^[A-Z]{4,4}(?:-\d\d\d\d){4,4}$').match(value):
+        raise ValidationError(
+            '%(value)s is not a valid Satellite Identifier. Satellite Identifier should have the \
+             format "CCCC-NNNN-NNNN-NNNN-NNNN" where C is {A-Z} and N is {0-9}',
+            params={'value': value},
+        )
+
+
+def generate_sat_id():
+    """Generate Satellite Identifier"""
+    numeric = "0123456789"
+    uppercase = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
+    first_segment = generate(uppercase, 4)
+    second_segment = generate(numeric, 4)
+    third_segment = generate(numeric, 4)
+    fourth_segment = generate(numeric, 4)
+    fifth_segment = generate(numeric, 4)
+    return "{0}-{1}-{2}-{3}-{4}".format(
+        first_segment, second_segment, third_segment, fourth_segment, fifth_segment
+    )
+
+
+class SatelliteIdentifier(models.Model):
+    """Model for Satellite Identifier."""
+    sat_id = models.CharField(
+        default=generate_sat_id, unique=True, max_length=24, validators=[validate_sat_id]
+    )
+    created = models.DateTimeField(auto_now_add=True)
+
+    def __str__(self):
+        return '{0}'.format(self.sat_id)
+
+
+class SatelliteEntry(models.Model):
+    """Model for all the satellite entries."""
+    satellite_identifier = models.ForeignKey(
+        SatelliteIdentifier, null=True, related_name='satellite_entries', on_delete=models.PROTECT
+    )
+    norad_cat_id = models.PositiveIntegerField(blank=True, null=True)
     norad_follow_id = models.PositiveIntegerField(blank=True, null=True)
     name = models.CharField(max_length=45)
     names = models.TextField(blank=True)
@@ -113,9 +153,30 @@ class Satellite(models.Model):
         on_delete=models.SET_NULL,
         related_name='satellite_operator'
     )
+    # Fields related to suggestion reviews
+    reviewer = models.ForeignKey(
+        get_user_model(),
+        related_name='reviewed_satellites',
+        blank=True,
+        null=True,
+        on_delete=models.SET_NULL
+    )
+    reviewed = models.DateTimeField(blank=True, null=True, help_text='Timestamp of review')
+    approved = models.BooleanField(default=False)
+    created = models.DateTimeField(default=now, help_text='Timestamp of creation/edit')
+    created_by = models.ForeignKey(
+        get_user_model(), related_name='created_satellites', null=True, on_delete=models.SET_NULL
+    )
+    citation = models.CharField(
+        max_length=512,
+        default='CITATION NEEDED - https://xkcd.com/285/',
+        help_text='A reference (preferrably URL) for this entry or edit'
+    )
 
     class Meta:
         ordering = ['norad_cat_id']
+        verbose_name_plural = 'Satellite Entries'
+        unique_together = ("satellite_identifier", "reviewed")
 
     def get_description(self):
         """Returns the markdown-processed satellite description
@@ -135,14 +196,85 @@ class Satellite(models.Model):
             image = settings.SATELLITE_DEFAULT_IMAGE
         return image
 
+    @property
+    def countries_str(self):
+        """Returns countries for this Satellite in comma seperated string format
+
+        :returns: countries for this Satellite in comma seperated string format
+        """
+        return ','.join(map(str, self.countries))
+
+    def __str__(self):
+        return '{0} - {1}'.format(self.norad_cat_id, self.name)
+
+
+class SatelliteSuggestionManager(models.Manager):  # pylint: disable=R0903
+    """Django Manager for SatelliteSuggestions
+
+    SatelliteSuggestions are SatelliteEntry objects that have been
+    submitted (suggested) but not yet reviewed
+    """
+    def get_queryset(self):  # pylint: disable=R0201
+        """Returns SatelliteEntries that have not been reviewed"""
+        return SatelliteEntry.objects.filter(reviewed__isnull=True)
+
+
+class SatelliteSuggestion(SatelliteEntry):
+    """Proxy model for unreviewed SatelliteEntry objects"""
+    objects = SatelliteSuggestionManager()
+
+    class Meta:
+        proxy = True
+        permissions = (
+            ('approve_satellitesuggestion', 'Can approve/reject satellite suggestions'),
+        )
+
+
+class SatelliteManager(models.Manager):  # pylint: disable=R0903
+    """Django Manager for Satellites
+
+    Satellite objects rarely used without referencing their SatelliteIdentifier
+    and SatelliteEntry foreign key fields, thus follow these relationships to
+    get the related-objects data by using select_related().
+    """
+    def get_queryset(self):  # pylint: disable=R0201
+        """Returns SatelliteEntries that have not been reviewed"""
+        return super().get_queryset().select_related('satellite_identifier', 'satellite_entry')
+
+
+class Satellite(models.Model):
+    """ Model for the lastest satellite entry for each Satellite Identifier."""
+    objects = SatelliteManager()
+
+    satellite_identifier = models.OneToOneField(
+        SatelliteIdentifier, related_name='satellite', on_delete=models.CASCADE
+    )
+    satellite_entry = models.ForeignKey(SatelliteEntry, null=True, on_delete=models.SET_NULL)
+    associated_satellite = models.ForeignKey(
+        'self', null=True, related_name='associated_with', on_delete=models.PROTECT
+    )
+    last_modified = models.DateTimeField(auto_now=True)
+
+    class Meta:
+        permissions = (('merge_satellites', 'Can merge satellites'), )
+
+    def __str__(self):
+        if self.satellite_entry:
+            name = self.satellite_entry.name
+            norad_cat_id = self.satellite_entry.norad_cat_id
+        else:
+            name = '-'
+            norad_cat_id = '-'
+        return '{1} ({2}) | {0}'.format(self.satellite_identifier.sat_id, name, norad_cat_id)
+
     @property
     def transmitters(self):
         """Returns valid transmitters for this Satellite
 
         :returns: the valid transmitters for this Satellite
         """
-        transmitters = Transmitter.objects.filter(satellite=self.id)
-        return transmitters.exclude(status='invalid')
+        transmitters = Transmitter.objects.filter(satellite=self.id).exclude(status='invalid')
+        return transmitters
 
     @property
     def transmitter_suggestion_count(self):
@@ -153,6 +285,17 @@ class Satellite(models.Model):
         pending_count = TransmitterSuggestion.objects.filter(satellite=self.id).count()
         return pending_count
 
+    @property
+    def satellite_suggestion_count(self):
+        """Returns number of pending satellite suggestions for this Satellite
+
+        :returns: number of pending satellite suggestions for this Satellite
+        """
+        pending_count = SatelliteSuggestion.objects.filter(
+            satellite_identifier=self.satellite_identifier
+        ).count()
+        return pending_count
+
     @property
     def telemetry_data_count(self):
         """Returns number of DemodData for this Satellite
@@ -163,7 +306,9 @@ class Satellite(models.Model):
         if cached_satellite:
             data_count = cached_satellite['count']
         else:
-            data_count = DemodData.objects.filter(satellite=self.id).count()
+            data_count = DemodData.objects.filter(
+                Q(satellite=self.id) | Q(satellite__associated_satellite=self.id)
+            ).count()
         return data_count
 
     @property
@@ -175,31 +320,26 @@ class Satellite(models.Model):
         decoder_count = Telemetry.objects.filter(satellite=self.id).exclude(decoder='').count()
         return decoder_count
 
-    @property
-    def transmitter_count(self):
-        """Returns number of approved transmitter suggestions for this Satellite
-
-        :returns: number of approved transmitter suggestions for this Satellite
-        """
-        approved_count = Transmitter.objects.filter(satellite=self.id).count()
-        return approved_count
-
     @property
     def latest_data(self):
         """Returns the latest DemodData for this Satellite
 
         :returns: dict with most recent DemodData for this Satellite
         """
-        data = DemodData.objects.filter(satellite=self.id).order_by('-id')[:1]
-        latest_datum = data[0]
-        return {
-            'data_id': latest_datum.data_id,
-            'payload_frame': latest_datum.payload_frame,
-            'timestamp': latest_datum.timestamp,
-            'is_decoded': latest_datum.is_decoded,
-            'station': latest_datum.station,
-            'observer': latest_datum.observer,
-        }
+        data = DemodData.objects.filter(
+            Q(satellite=self.id) | Q(satellite__associated_satellite=self.id)
+        ).order_by('-id')[:1]
+        if data:
+            latest_datum = data[0]
+            return {
+                'data_id': latest_datum.data_id,
+                'payload_frame': latest_datum.payload_frame,
+                'timestamp': latest_datum.timestamp,
+                'is_decoded': latest_datum.is_decoded,
+                'station': latest_datum.station,
+                'observer': latest_datum.observer,
+            }
+        return None
 
     @property
     def needs_help(self):
@@ -209,19 +349,19 @@ class Satellite(models.Model):
         :returns: bool
         """
         score = 0
-        if self.description and self.description != '':
+        if self.satellite_entry.description and self.satellite_entry.description != '':
             score += 1
-        if self.countries and self.countries != '':
+        if self.satellite_entry.countries and self.satellite_entry.countries != '':
             score += 1
-        if self.website and self.website != '':
+        if self.satellite_entry.website and self.satellite_entry.website != '':
             score += 1
-        if self.names and self.names != '':
+        if self.satellite_entry.names and self.satellite_entry.names != '':
             score += 1
-        if self.launched and self.launched != '':
+        if self.satellite_entry.launched and self.satellite_entry.launched != '':
             score += 1
-        if self.operator and self.operator != '':
+        if self.satellite_entry.operator and self.satellite_entry.operator != '':
             score += 1
-        if self.image and self.image != '':
+        if self.satellite_entry.image and self.satellite_entry.image != '':
             score += 1
 
         return score <= 2
@@ -240,9 +380,6 @@ class Satellite(models.Model):
                 bad_transmitter_count += 1
         return bad_transmitter_count > 0
 
-    def __str__(self):
-        return '{0} - {1}'.format(self.norad_cat_id, self.name)
-
 
 class TransmitterEntry(models.Model):
     """Model for satellite transmitters."""
@@ -453,7 +590,9 @@ class TransmitterSuggestion(TransmitterEntry):
 
     class Meta:
         proxy = True
-        permissions = (('approve', 'Can approve/reject transmitter suggestions'), )
+        permissions = (
+            ('approve_transmittersuggestion', 'Can approve/reject transmitter suggestions'),
+        )
 
 
 class TransmitterManager(models.Manager):  # pylint: disable=R0903
@@ -470,7 +609,8 @@ class TransmitterManager(models.Manager):  # pylint: disable=R0903
         ).filter(uuid=OuterRef('uuid')).order_by('-reviewed')
         return super().get_queryset().filter(
             reviewed__isnull=False, approved=True
-        ).filter(reviewed=Subquery(subquery.values('reviewed')[:1]))
+        ).select_related('satellite', 'downlink_mode',
+                         'uplink_mode').filter(reviewed=Subquery(subquery.values('reviewed')[:1]))
 
 
 class Transmitter(TransmitterEntry):
@@ -503,7 +643,7 @@ class Tle(models.Model):
     tle_source = models.CharField(max_length=300, blank=True)
     updated = models.DateTimeField(auto_now=True, blank=True)
     satellite = models.ForeignKey(
-        Satellite, related_name='tles', on_delete=models.SET_NULL, null=True, blank=True
+        Satellite, null=True, blank=True, related_name='tle_sets', on_delete=models.SET_NULL
     )
 
     class Meta:
@@ -545,7 +685,7 @@ class Telemetry(models.Model):
     decoder = models.CharField(max_length=200, blank=True)
 
     class Meta:
-        ordering = ['satellite__norad_cat_id']
+        ordering = ['satellite__satellite_entry__norad_cat_id']
         verbose_name_plural = 'Telemetries'
 
     def __str__(self):
@@ -597,7 +737,7 @@ class DemodData(models.Model):
         ordering = ['-timestamp']
 
     def __str__(self):
-        return 'data-for-{0}'.format(self.satellite.norad_cat_id)
+        return 'data-for-{0}'.format(self.satellite.satellite_identifier.sat_id)
 
     def display_frame(self):
         """Returns the contents of the saved frame file for this DemodData
