@@ -456,14 +456,18 @@ def cache_statistics():
             if sat['associated_satellite'] in sat_stats:
                 sat_stats[sat['associated_satellite']]['count'] += sat['count']
                 sat_stats[sat['associated_satellite']]['decoded'] += sat['decoded']
-                sat_stats[sat['associated_satellite']]['latest_payload'] = max(
-                    sat_stats[sat['associated_satellite']]['latest_payload'],
-                    sat['latest_payload'],
-                    default=sat['latest_payload']
-                )
+                if sat['latest_payload']:
+                    sat_stats[sat['associated_satellite']]['latest_payload'] = max(
+                        [
+                            sat_stats[sat['associated_satellite']]['latest_payload'],
+                            sat['latest_payload']
+                        ],
+                        default=sat['latest_payload']
+                    )
             else:
+                sat_id = sat_stats[sat['associated_satellite']]
                 del sat['associated_satellite']
-                sat_stats[sat['associated_satellite']] = sat
+                sat_stats[sat_id] = sat
         else:
             ids.append(sat['id'])
             # if non-merged satellite is already in sat_stats then overwrite
