@@ -51,7 +51,8 @@ def home(request):
     latest_data_satellites = []
     found = False
     date_from = now() - timedelta(days=1)
-    data_list = DemodData.objects.filter(timestamp__gte=date_from).order_by('-pk')
+    data_list = DemodData.objects.filter(timestamp__gte=date_from
+                                         ).order_by('-pk').select_related('satellite')
     paginator = Paginator(data_list, 50)
     page = paginator.page(1)
     while not found:
